SITE_NAME = "Abilian SBE Dev"
TESTING = False
PRODUCTION = False
CSRF_ENABLED = True
SECRET_KEY = 'CHANGEME'

PREFERRED_URL_SCHEME = 'http'

DEBUG = True
DEBUG_TOOLBAR = True
DEBUG_TB_INTERCEPT_REDIRECTS = False
DEBUG_TB_PROFILER_ENABLED = True

LOGGING_CONFIG_FILE = 'logging.yml'

SQLALCHEMY_DATABASE_URI = "sqlite:///INSTANCE_FOLDER/data.db"
# SQLALCHEMY_DATABASE_URI = postgres://user@localhost:5432/abilian_sbe
# SQLALCHEMY_ECHO = True

# run tasks locally, no async
CELERY_ALWAYS_EAGER = True
CELERY_EAGER_PROPAGATES_EXCEPTIONS = True

MAIL_SENDER = "user@localhost"
MAIL_FAIL_SILENTLY = False
