<html>
<body>

<h1>Please confirm your request to unsubscribe from communities daily notifications</h1>

<p>
  After clicking the confirm button below, you will not receive another daily notification email.
</p>

<form action="." method="POST">
  <button>Confirm Unsubscribe</button>
</form>

</body>
</html>
